/*
 *   libdanceracpi - Dancer's ACPI monitoring system modules
 *   Copyright (C) 2002 Junichi Uekawa
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include "libdanceracpi/libdanceracpi.h"
#include "libdshconfig.h"

int 
dacpi_getbatterystatus(int battery_id, dacpi_battery_status * st)
{
  char * procpath = NULL;
  FILE * f;
  dshconfig * d = NULL ;
  dshconfig_internal * i;
  
  if (asprintf (&procpath, "/proc/acpi/battery/%i/status", battery_id)<0)
    return 1;
  
  f = fopen (procpath, "r");
  if (!f) return 2;
  free (procpath);
  d = open_dshconfig (f, ':');
  if (!d)
    return 1;
  for (i=d->config; i; i=i->next)
    {
      if (!strcmp (i->title, "Present Rate"))
	st->rate = atol (i->data);
      else if (!strcmp (i->title, "Remaining Capacity"))
	st->capacity = atol (i->data);
      else if (!strcmp (i->title, "Battery Voltage"))
	st->voltage = atol (i->data);
    }
  
  free_dshconfig (d);
  fclose (f);

  st->api_version = DACPI_API_VERSION;
  
  return 0;
}

int 
dacpi_getbatteryinfo(int battery_id, dacpi_battery_info * info)
{
  char * procpath = NULL;
  FILE * f;
  dshconfig * d;
  dshconfig_internal * di;
  
  if (asprintf (&procpath, "/proc/acpi/battery/%i/info", battery_id)<0)
    return 1;
  
  f = fopen (procpath, "r");
  if (!f) return 2;
  free (procpath);
  d = open_dshconfig(f, ':');
  if (!d)
    {
      fclose(f);
      return 1;
    }
  
  for (di = d -> config; di; di=di->next)
    {
      if (!strcmp (di->title, "Design Capacity"))
	info -> designcapacity = atol (di->data);
      else if (!strcmp (di->title, "Last Full Capacity"))
	info -> lastfullcapacity = atol (di->data);
      else if (!strcmp (di->title, "Design Voltage"))
	info -> designvoltage = atol (di->data);
      else if (!strcmp (di->title, "Design Capacity Warning"))
	info -> designcapacitywarning = atol (di->data);
      else if (!strcmp (di->title, "Design Capacity Low"))
	info -> designcapacitylow = atol (di->data);
      else if (!strcmp (di->title, "Capacity Granularity 1"))
	info -> capacitygranularity1 = atol (di->data);
      else if (!strcmp (di->title, "Capacity Granularity 2"))
	info -> capacitygranularity2 = atol (di->data);

    }
  
  free_dshconfig(d);
  fclose(f);
  
  info->api_version = DACPI_API_VERSION;
  return 0;
}


int
dacpi_get_battery ( int battery_id, dacpi_battery * b)
{
  dacpi_battery_status m;
  dacpi_battery_info i;
  
  dacpi_getbatterystatus(battery_id, &m);
  dacpi_getbatteryinfo(battery_id, &i);

  b -> battery_max = i.lastfullcapacity;
  b -> battery_using = m.rate;
  b -> battery_current = m.capacity;
  b -> api_version = DACPI_API_VERSION;
  
  return 0;
}

DACPI_ADAPTER_TYPE
dacpi_get_ac_adapter (int ac_id)
{
  char * procpath = NULL;
  FILE * f;
  const char * onlinestring = NULL;
  dshconfig * d;
  int return_value;

  if (asprintf (&procpath, "/proc/acpi/ac_adapter/%i/status", ac_id)<0)
    return DACPI_AC_ERROR;
  
  f = fopen (procpath, "r");
  if (!f) return DACPI_AC_ERROR;
  free (procpath);
  d = open_dshconfig (f, ':');
  onlinestring = dshconfig_searchdata(d, "Status");

  if (!strcmp (onlinestring, "on-line"))
    return_value = DACPI_AC_ONLINE;
  else if (!strcmp (onlinestring, "off-line"))
    return_value = DACPI_AC_OFFLINE;
  else
    return_value = DACPI_AC_ERROR;

  free_dshconfig(d);  
  fclose(f);

  return return_value;
}

long
dacpi_get_battery_using (int battery_id)
{
  dacpi_battery b;
  dacpi_get_battery (battery_id, &b);
  return b.battery_using;
}


long
dacpi_get_battery_max (int battery_id)
{
  dacpi_battery b;
  dacpi_get_battery (battery_id, &b);
  return b.battery_max;
}

long
dacpi_get_battery_current (int battery_id)
{
  dacpi_battery b;
  dacpi_get_battery (battery_id, &b);
  return b.battery_current;
}

int 
dacpi_check_acpi_support (void)
{
  struct stat s;
  return 0==stat ("/proc/acpi/", &s);
}

int
dacpi_obtain_library_api_version (void)
{
  return DACPI_API_VERSION;
}
